/*  -r1.11 sdlcmain.c
    @(#) module: sdlcmain.c, version: 1.11, retrieved: 01/12/04 18:37:39 */

/*
 *  Vers Description:											Date:	   By:
 * ===== ====================================================== ========== ====
 *  1.9  Max. number of devices increased to 32.				10-07-2001 jfk
 *		 Default tty-path is Operating System Dependent.
 *	1.11 Support for industrial codes; adapted to 'buildsd'.	28-11-2001 jfk
 */

#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <string.h>
#include <strings.h>
#include <sys/stat.h>
#include <unistd.h>
#include <ctype.h>
#include <fcntl.h>
#include <poll.h>
#include <signal.h>
#include <time.h>
#include "sdlcmain.h"
#include "crcframe.h"

char *UsageMsg[] = {
	"example(sco): sd -l /dev/tty1a -n4"
};



#define SZADDRESS	4		/* size of address string, whitout null byte */

struct DeviceDscr {
	BYTE TxFrame[FRAMSIZ];
	BYTE RxFrame[FRAMSIZ];
	BYTE PollFrame[4];
	WORD State;
	unsigned int ArtRequest;
	unsigned int ErrorCnt;
	BYTE DeviceAddr;
	BYTE DeviceNr;
	BYTE DeviceType;
	BYTE SSS;
	BYTE RRR;
	BYTE PollTimeOut;
	BYTE TimeOutCnt;
	BYTE PollError;
};

int Debug, Signals, PollTimout, FiFoTimout;
int fdoutfifo, fdinfifo, fdpinfifo;
BYTE SGModus;
FILE *fplog;

extern void SendNSFrame(BYTE NSType, struct DeviceDscr *Device, BYTE SysCmd);
extern char *ctime_r(const time_t *clock, char *buf);

const char VersionMsg[] =
"@(#) Module:sd Version:" SWRNUMBER "-" PLATFORM " Date:" BUILDDATE ;

char *UnixOSTab[] = { "SunOS5.", "SCO_SV3", "LINUX2.", NULL };
char *ttypathTab[] = { "/dev/ttyb", "/dev/tty1a", "/dev/cua0", "/dev/unknown"};

const char *DeviceTb[] = {"SG10", "SG20"};
const char DBOffLine[] = " \x1b'00No server response";

char *ErrorMsg[] = {"E_RXCRC",
					"E_MAXRETRY",
					"E_UNKNWN",
					};

/*
 * Use second least significant byte to trace the code section
 * where the error showed up.
 */ 
char *TraceMsg[] = {"POLLFRAME_1",
					"SENDICMD_1",
					};

BYTE *ConvertFrame2String(register BYTE *s, register BYTE *d)
{
	register BYTE *End=s;

	return(End);
}

/*
 * Error Format: Signbit(31)...Error(15-8)-TracePoint(7-0).
 */
int HandleError(struct DeviceDscr *pl, int Error)
{

	return(Error);
}

void SignalHandler(int ReceivedSignal)
{
	Signals |= ReceivedSignal;
}

void tsleep(unsigned int nmsec)
{
	struct pollfd dummy;

	if (nmsec)
		poll(&dummy, 0, nmsec);
}

void SendNSFrame(BYTE NSType, struct DeviceDscr *Device, BYTE SysCmd) 
{
	BYTE Frame[] = "    ";

	CRCFrame(Frame, XMIT);			/* add CRC and terminate with End Of Frame Flag */
	sdlc(sdSWRITE, Frame);
}

BYTE AddressTab[] = {	0x7a, 0x70, 0x71, 0x72, 0x73, 0x7d, 0x7b };

void InitDeviceList(struct DeviceDscr *Listp, int n)
{
	register BYTE i, j, *s;
	register struct DeviceDscr *pl;

	for (i=0, s=AddressTab, pl=Listp; i <= (BYTE)(n); i++)
		j = *s++;
}

int SendRRFrame(register struct DeviceDscr *dd)
{
	BYTE Frame[] = "   ";

	CRCFrame(Frame, XMIT);		/* add CRC and terminate with End Of Frame Flag */
	return(sdlc(sdSWRITE, Frame)); 
}

int PollFrame(struct DeviceDscr *pl)
{

	return(1);
}

int SendIFrameAck(register struct DeviceDscr *pl, BYTE Data[], WORD WaitIFrame)
{

	return(5);
}

int SendICmd(struct DeviceDscr *dd, BYTE Cmd)
{

	return(0);
}

int CheckROLDevice(struct DeviceDscr *dd)
{
	int i;

	i = 0;
	return(i);
}

int FormatLog(BYTE Buf[], register BYTE *s, int j)
{
	return(j);
}

int PutMsg(struct DeviceDscr *dd, BYTE Buf[], struct pollfd *PollList)
{
	return(0);
}

BYTE *Checkpinfifo(BYTE Buf[], FILE *fd)
{
	return(Buf);
}

void OnLineMsg(register struct DeviceDscr *pl)
{
}

int ProcessList(struct DeviceDscr PollList[], int n)
{
	return(0);
}

void ShowUsage(char *cp)
{
}

int CheckFIFO(char fifoName[], int Mode)
{
	return(Mode);
}

int ParseCmdLine(char **ArgV, int ArgCnt, int *n, struct SDConfig *sdConfig)
{
	return(*n);
}

int main(int ArgCnt, char *Argv[])
{
	struct SDConfig sdConfig;
	struct DeviceDscr *PollList;
	int n;
	time_t Time;

	if (ParseCmdLine(Argv, ArgCnt, &n, &sdConfig) > 0) {
		if (sdlc(sdOPEN, (BYTE *)(&sdConfig)) >= 0) {
			if ((PollList=malloc((n+4)*sizeof(struct DeviceDscr))) != NULL) {
				Signals = 0;
				signal(SIGINT, SignalHandler);
				ProcessList(PollList, n);
				fflush(fplog);
				sdlc(sdCLOSE, (BYTE *)(&sdConfig));
				free(PollList);
			} else printf("Fatal: could not allocate enough memory\n");
		} else fprintf(fplog, "%s", sdConfig.devpath);
		close(fdoutfifo);
		close(fdinfifo);
		if (fplog != stdout) {
			Time = time(NULL);
			fprintf(fplog, "Closing logfile: %s", ctime(&Time));
			fclose(fplog);
		}
	}
	return(0);
}
